/**************************************************************************\
*          Copyright (c) 1995 by Stichting Mathematisch Centrum,           *
*                                Amsterdam, The Netherlands.               *
*                                                                          *
*                           All Rights Reserved                            *
*                                                                          *
* Permission to use, copy, modify, and distribute this software and its    *
* documentation for any purpose and without fee is hereby granted,         *
* provided that the above copyright notice appear in all copies and that   *
* both that copyright notice and this permission notice appear in          *
* supporting documentation, and that the names of Stichting Mathematisch   *
* Centrum or CWI not be used in advertising or publicity pertaining to     *
* distribution of the software without specific, written prior permission. *
*                                                                          *
* STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO   *
* THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND   *
* FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE      *
* FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES        *
* WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN    *
* ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT     *
* OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.        *
*                                                                          *
\**************************************************************************/
/**************************************************************************\
* 	                						   *
*  File              : videosgi.c                                          *
*  Author            : Guido van Rossum, CWI, Amsterdam.                   *
*  Last Modification : 1995/2/15                                           *
*--------------------------------------------------------------------------*
*  Description       : Video interface for SGI Indigo.                     *
*                                                                          *
*--------------------------------------------------------------------------*
*        Name	        |    Date   |          Modification                *
*--------------------------------------------------------------------------*
* Thierry Turletti      |  93/6/28  | Adaptation to the 3.2 version.       *
*--------------------------------------------------------------------------*
* Lei Wang              |  93/6/30  | Color added                          *   
\**************************************************************************/
#ifdef INDIGOVIDEO

#include <stdio.h>
#include <svideo.h>

#include "h261.h"
#include "protocol.h"

#define LIG_MAX			288	/* Max number of lines and columns */
#define COL_MAX			352

#define LIG_MAX_DIV2		144	
#define COL_MAX_DIV2		176

#define CIF_WIDTH               352     /* Width and height of CIF */
#define CIF_HEIGHT		288

#define QCIF_WIDTH              176     /* Width and height of Quarter CIF */
#define QCIF_HEIGHT		144

#define MODCOLOR		4
#define MODGREY			2
#define MODNB			1

/*
**		IMPORTED VARIABLES
*/

/*-------- from display.c --------*/
extern BOOLEAN	CONTEXTEXIST;
extern BOOLEAN	encCOLOR;
extern u_char	affCOLOR;

/*
**		GLOBAL VARIABLES
*/

XtAppContext  appCtxt;
Widget        appShell;
Display       *dpy;
XImage        *ximage;
GC            gc;
Window        win;
Window        icon;
XColor        colors[256];
int           depth;

void	build_image();
void    build_icon_image();

/* table to map SGI 8 bits RGB to CCIR YUV */

static u_char SGI_RGB8_TO_CCIR_Y[256] = {
  16,  34,  53,  72,  90, 109, 128, 146,  23,  41,  
  60,  79,  98, 116, 135, 154,  30,  49,  67,  86, 
  105, 123, 142, 161,  37,  56,  75,  93, 112, 131, 
  149, 168,  25,  44,  62,  81, 100, 118, 137, 156,  
  32,  51,  70,  88, 107, 126, 144, 163,  40,  58,  
  77,  96, 114, 133, 152, 170,  47,  66,  84, 103, 
  122, 140, 159, 178,  35,  53,  72,  91, 109, 128, 
  147, 165,  42,  61,  79,  98, 117, 135, 154, 173,  
  49,  68,  86, 105, 124, 143, 161, 180,  56,  75,  
  94, 112, 131, 150, 168, 187,  44,  63,  81, 100, 
  119, 138, 156, 175,  51,  70,  89, 107, 126, 145, 
  163, 182,  59,  77,  96, 115, 133, 152, 171, 189,  
  66,  85, 103, 122, 141, 159, 178, 197,  54,  72, 
  91, 110, 128, 147, 166, 184,  61,  80,  98, 117, 
  136, 154, 173, 192,  68,  87, 106, 124, 143, 162, 
  180, 199,  75,  94, 113, 131, 150, 169, 188, 206,  
  63,  82, 101, 119, 138, 157, 175, 194,  70,  89, 
  108, 126, 145, 164, 183, 201,  78,  96, 115, 134, 
  152, 171, 190, 208,  85, 104, 122, 141, 160, 178, 
  197, 216,  73,  91, 110, 129, 147, 166, 185, 203,  
  80,  99, 117, 136, 155, 173, 192, 211,  87, 106, 
  125, 143, 162, 181, 199, 218,  94, 113, 132, 151, 
  169, 188, 207, 225,  82, 101, 120, 138, 157, 176, 
  194, 213,  89, 108, 127, 146, 164, 183, 202, 220,  
  97, 115, 134, 153, 171, 190, 209, 228, 104, 123, 
  141, 160, 179, 197, 216, 235 };

static u_char SGI_RGB8_TO_CCIR_UDIV4[256] = {
  33, 31, 28, 25, 22, 19, 17, 14, 42, 39, 
  36, 33, 31, 28, 25, 22, 50, 48, 45, 42,
  39, 36, 34, 31, 59, 56, 53, 50, 48, 45, 
  42, 39, 32, 29, 26, 24, 21, 18, 15, 12,
  40, 38, 35, 32, 29, 26, 24, 21, 49, 46, 
  43, 41, 38, 35, 32, 29, 57, 55, 52, 49,
  46, 43, 41, 38, 31, 28, 25, 22, 19, 17, 
  14, 11, 39, 36, 33, 31, 28, 25, 22, 19,
  47, 45, 42, 39, 36, 33, 31, 28, 56, 53, 
  50, 48, 45, 42, 39, 36, 29, 26, 23, 21,
  18, 15, 12,  9, 38, 35, 32, 29, 26, 24, 
  21, 18, 46, 43, 40, 38, 35, 32, 29, 26, 
  55, 52, 49, 46, 43, 41, 38, 35, 28, 25, 
  22, 19, 16, 14, 11,  8, 36, 33, 31, 28, 
  25, 22, 19, 17, 45, 42, 39, 36, 33, 31, 
  28, 25, 53, 50, 48, 45, 42, 39, 36, 34, 
  26, 23, 21, 18, 15, 12,  9,  7, 35, 32, 
  29, 26, 24, 21, 18, 15, 43, 40, 38, 35, 
  32, 29, 26, 24, 52, 49, 46, 43, 40, 38, 
  35, 32, 25, 22, 19, 16, 14, 11,  8,  5, 
  33, 30, 28, 25, 22, 19, 16, 14, 42, 39, 
  36, 33, 31, 28, 25, 22, 50, 47, 45, 42, 
  39, 36, 33, 31, 23, 21, 18, 15, 12,  9, 
  7,  4, 32, 29, 26, 23, 21, 18, 15, 12, 
  40, 38, 35, 32, 29, 26, 24, 21, 49, 46, 
  43, 40, 38, 35, 32, 29
};

static u_char SGI_RGB8_TO_CCIR_VDIV4[256] = {
  31, 28, 24, 21, 18, 14, 11,  8, 30, 26, 
  23, 20, 16, 13, 10,  6, 28, 25, 22, 18,
  15, 12,  8,  5, 27, 24, 20, 17, 14, 10, 
  7,  4, 35, 32, 28, 25, 22, 18, 15, 12, 
  34, 30, 27, 24, 20, 17, 14, 10, 32, 29, 
  26, 22, 19, 16, 12,  9, 31, 28, 24, 21, 
  18, 14, 11,  8, 39, 36, 32, 29, 26, 22, 
  19, 16, 38, 34, 31, 28, 24, 21, 18, 14, 
  36, 33, 30, 26, 23, 20, 16, 13, 35, 32, 
  28, 25, 22, 18, 15, 12, 43, 40, 36, 33, 
  30, 26, 23, 20, 42, 38, 35, 32, 28, 25, 
  22, 18, 40, 37, 34, 30, 27, 24, 20, 17, 
  39, 36, 32, 29, 26, 22, 19, 16, 47, 44, 
  40, 37, 34, 30, 27, 24, 46, 42, 39, 36,
  32, 29, 26, 22, 44, 41, 38, 34, 31, 28, 
  24, 21, 43, 40, 36, 33, 30, 26, 23, 20,
  51, 48, 44, 41, 38, 34, 31, 28, 50, 46, 
  43, 40, 36, 33, 30, 26, 48, 45, 42, 38, 
  35, 32, 28, 25, 47, 44, 40, 37, 34, 30, 
  27, 24, 55, 52, 48, 45, 42, 38, 35, 32, 
  54, 50, 47, 44, 40, 37, 34, 30, 52, 49, 
  46, 42, 39, 36, 32, 29, 51, 48, 44, 41, 
  38, 34, 31, 28, 59, 56, 52, 49, 45, 42, 
  39, 35, 58, 54, 51, 48, 44, 41, 38, 34, 
  56, 53, 50, 46, 43, 40, 36, 33, 55, 52, 
  48, 45, 42, 38, 35, 32
};

/* sgi_checkvideo() -- return 1 if video capture is possible, 0 if not */

int sgi_checkvideo() {
  SVhandle dev;
  
  /* Open the video device.  If this fails, assume we don't have video */
  dev = svOpenVideo();
  if (dev == NULL) {
    svPerror("sgi_checkvideo: svOpenVideo");
    return 0;
  }
  /* Close the device again; sgi_video_encode will re-open it */
  svCloseVideo(dev);
  return 1;
}

/* sgi_get_image() -- grab a CIF image */
static void sgi_get_image(dev, new_Y, new_Cb, new_Cr, lig, col, w, h)
     SVhandle dev;
     u_char new_Y[LIG_MAX][COL_MAX];
     u_char new_Cb[LIG_MAX_DIV2][COL_MAX_DIV2];
     u_char new_Cr[LIG_MAX_DIV2][COL_MAX_DIV2];
     int lig, col;
     int w, h;
{
  char *ptr;
  long field;
  register u_char *p_data_y, *p_data_u, *p_data_v;
  register char *p_buf;
  register int j;
  char *buf_even;
  char *buf_odd;
  int i;
  int xoff, yoff;
  
  while(1){
    ptr = NULL;
    if(svGetCaptureData(dev, (void **)&ptr, &field) < 0){
      svPerror("sgi_get_image: svGetCaptureData");
      exit(1);
    }
    if(ptr != NULL)
      break;
    sginap(1);
  }
  
  /* The capture area is larger than col*lig; use the center */
  xoff = (w-col)/2;
  yoff = (h-lig)/2;
  
  buf_odd = &ptr[(yoff/2)*w + xoff];
  buf_even = &ptr[(yoff/2 + h/2)*w + xoff];
  
  for(i = 0; i < lig; i++){
    p_data_y = &new_Y[i][0];
    if((i&1) == 0){
      p_buf = &buf_even[(i/2)*w];
      p_data_u = &new_Cb[(i/2)][0];
      p_data_v = &new_Cr[(i/2)][0];
      for(j = 0; j < col; j++){
	*p_data_y++ = SGI_RGB8_TO_CCIR_Y[*p_buf++];
	if((j&1) == 0){
	  *p_data_u = SGI_RGB8_TO_CCIR_UDIV4[*p_buf];
	  *p_data_v = SGI_RGB8_TO_CCIR_VDIV4[*p_buf];
	}else{
	  *p_data_u += SGI_RGB8_TO_CCIR_UDIV4[*p_buf];
	  *p_data_v += SGI_RGB8_TO_CCIR_VDIV4[*p_buf];
	  *p_data_u++;
	  *p_data_v++;
	}
      }
    }
    else{
      p_buf = &buf_odd[(i/2)*w];
      p_data_u = &new_Cb[((i-1)/2)][0];
      p_data_v = &new_Cr[((i-1)/2)][0];
      for(j = col; --j >= 0; ){
	*p_data_y++ = SGI_RGB8_TO_CCIR_Y[*p_buf++];
	if((j&1) == 0){
	  *p_data_u += SGI_RGB8_TO_CCIR_UDIV4[*p_buf];
	  *p_data_v += SGI_RGB8_TO_CCIR_VDIV4[*p_buf];
	}else{
	  *p_data_u += SGI_RGB8_TO_CCIR_UDIV4[*p_buf];
	  *p_data_v += SGI_RGB8_TO_CCIR_VDIV4[*p_buf];
	  *p_data_u++;
	  *p_data_v++;
	}
      }
    }
  }
  svUnlockCaptureData(dev, ptr);
}

/* sgi_video_encode() -- the video capture, coding and tranmission process */

void sgi_video_encode(s_video)
     
     S_VIDEO *s_video;
     
{
  IMAGE_CIF image_coeff;
  u_char new_Y[LIG_MAX][COL_MAX];
  u_char new_Cb[LIG_MAX_DIV2][COL_MAX_DIV2];
  u_char new_Cr[LIG_MAX_DIV2][COL_MAX_DIV2];
  
  int NG_MAX;
  BOOLEAN CONTINUE=TRUE;
  BOOLEAN LOCAL_DISPLAY, TUNING;
  BOOLEAN ICONIC_DISPLAY=FALSE;
  int lig, col;
  int w, h;
  
  SVhandle dev;
  svCaptureInfo info;
  long params[4];
  
  /* Open video device */
  dev = svOpenVideo();
  if (dev == NULL) {
    fprintf(stderr, "sgi_video_encode: svOpenVideo failed\n");
    svPerror("sgi_video_encode: svOpenVideo");
    exit(1);
  }
  
  /* Set the state of video variables */ 
  params[0] = SV_COLOR;
  params[2] = SV_DITHER;
  if(!s_video->COLOR){ 
    params[1] = SV_MONO;
    params[3] = FALSE;
  }else{
    params[1] = SV_USER_COLOR;
    params[3] = TRUE;
  }
  
  if(svSetParam(dev, params, 4) < 0){
    svPerror("sgi_video_encode: svSetParam()");
  }
  
  if(!s_video->COLOR){
    encCOLOR = FALSE;
    affCOLOR = MODGREY;
  }else{
    encCOLOR = TRUE;
    affCOLOR = MODCOLOR;
  }
  
  if(s_video->video_size == QCIF_SIZE){
    NG_MAX = 5;
    lig = QCIF_HEIGHT;
    col = QCIF_WIDTH;
  }else{
    NG_MAX = 12;
    lig = CIF_HEIGHT;
    col = CIF_WIDTH;
  }
  
  w = col;
  h = lig;
  if(w*3/4 < h)
    w = h*4/3;
  if(h*4/3 < w)
    h = w*3/4;
  svQuerySize(dev, w, h, &w, &h);
  svSetSize(dev, w, h);
  
  /* Initialize continuous capture mode */
  info.format = SV_RGB8_FRAMES;
  info.width = w;
  info.height = h;
  info.size = 1;
  info.samplingrate = 2;
  svInitContinuousCapture(dev, &info);
  w = info.width;
  h = info.height;
  if (w < col || h < lig) {
    fprintf(stderr, "sgi_encode_video: capture size too small\n");
    exit(1);
  }
  
  do{
    if(!FREEZE)
      sgi_get_image(dev, new_Y, new_Cb, new_Cr, lig, col, w, h);
    encode_h261(s_video, new_Y[0], new_Cb[0], new_Cr[0], image_coeff[0],
		NG_MAX);
    
    if(s_video->LOCAL_DISPLAY){
      build_image((u_char *) &new_Y[0][0], 
		  (u_char *) &new_Cb[0][0], 
		  (u_char *) &new_Cr[0][0], 
		  FALSE, ximage, COL_MAX, 0, col, lig);
      show(dpy, win, ximage);
    }
    LOCAL_DISPLAY = s_video->LOCAL_DISPLAY;
    if(CONTINUE = HandleCommands(s_video)){
      if(LOCAL_DISPLAY!= s_video->LOCAL_DISPLAY){
	if(s_video->LOCAL_DISPLAY){
	  if(CONTEXTEXIST) 
	    XtMapWidget(appShell);
	  else
	    init_window(lig, col, "Local display", s_video->display, 
			&appCtxt, &appShell, &depth, &dpy, &win,
			&icon, &gc, &ximage, FALSE);
	}else
	  if(CONTEXTEXIST){
	    if(ICONIC_DISPLAY){
	      ICONIC_DISPLAY =FALSE;
	      XUnmapWindow(dpy, icon);
	    }else
	      XtUnmapWidget(appShell);
	    while(XtAppPending (appCtxt))
	      XtAppProcessEvent (appCtxt, XtIMAll);	
	  }
      }
    }
    
/* looking for X events on win */
    if(s_video->LOCAL_DISPLAY){
      XEvent evt;
      while(XtAppPending (appCtxt)){
	XtAppNextEvent(appCtxt, &evt);
	switch(evt.type){
	case UnmapNotify:
	  if(evt.xunmap.window == icon){
	    ICONIC_DISPLAY = FALSE;
	  }
	  break;
	case MapNotify:
	  if(evt.xmap.window == win){
	    ICONIC_DISPLAY = FALSE;
	    break;
	  }
	  if(evt.xmap.window == icon){
	    ICONIC_DISPLAY = TRUE;
	    build_icon_image(icon, s_video->video_size, new_Y, new_Cb, new_Cr,
			     COL_MAX, col, lig, TRUE);
	  }
	  break;
	} /* hctiws evt.type */
	XtDispatchEvent (&evt) ;
      } /* elihw XtAppPending */
    } /* fi LOCAL_DISPLAY */
  }while(CONTINUE);
  
  /* Clean up and exit */
  close(s_video->f2s_fd[0]);
  close(s_video->s2f_fd[1]);
  if(s_video->LOCAL_DISPLAY)
    XCloseDisplay(dpy);
  svEndContinuousCapture(dev);
  svCloseVideo(dev);
  exit(0);
  /* NOT REACHED */
}
#endif /* INDIGOVIDEO */



